-- bulktiming.sql

DROP TABLE parts;

CREATE TABLE parts (
   partnum NUMBER,
   partname VARCHAR2(15)
   );
   
CREATE TABLE parts2 (
   partnum NUMBER,
   partname VARCHAR2(15)
   );
   
CREATE OR REPLACE PROCEDURE compare_inserting (num IN INTEGER)
IS
   TYPE NumTab IS TABLE OF parts.partnum%TYPE INDEX BY BINARY_INTEGER;
   TYPE NameTab IS TABLE OF parts.partname%TYPE INDEX BY BINARY_INTEGER;
   pnums NumTab;
   pnames NameTab;
BEGIN
   FOR indx IN 1..num LOOP
      pnums(indx) := indx;
      pnames(indx) := 'Part ' || TO_CHAR(indx);
   END LOOP;
   
   PLVtmr.capture;
   FOR indx IN 1..num LOOP
      INSERT INTO parts VALUES (pnums(indx), pnames(indx));
   END LOOP;
   PLVtmr.show_elapsed ('FOR loop '|| num);
   
   ROLLBACK;
   
   PLVtmr.capture;

   FORALL indx IN pnums.FIRST .. pnums.LAST
      INSERT INTO parts 
        VALUES (pnums(indx), pnames(indx));
   
   p.l ('SQL%ROWCOUNT = ' || SQL%ROWCOUNT);
   
   /*
   FOR indx IN 1..num
   LOOP
      SQL%BULK_ROWCOUNT(indx));
   END LOOP;
   */
   
   PLVtmr.show_elapsed ('FORALL '|| num);
   
   ROLLBACK;
END;
/

CREATE OR REPLACE PROCEDURE compare_fetching (num IN INTEGER)
IS
   TYPE NumTab IS TABLE OF parts.partnum%TYPE INDEX BY BINARY_INTEGER;
   TYPE NameTab IS TABLE OF parts.partname%TYPE INDEX BY BINARY_INTEGER;
   pnums NumTab;
   pnames NameTab;
   
   TYPE parts_tabtype IS TABLE OF parts%ROWTYPE 
      INDEX BY BINARY_INTEGER;
   parts_tab parts_tabtype;
   
   l_limit PLS_INTEGER := 100;
BEGIN
   EXECUTE IMMEDIATE 'TRUNCATE TABLE parts';
   
   /* Load up the table. */
   FOR indx IN 1..num
   LOOP
      INSERT INTO parts VALUES (indx, 'Part ' || TO_CHAR(indx));
   END LOOP;
   COMMIT;
   
   DBMS_SESSION.FREE_UNUSED_USER_MEMORY;
   
   /* Fetch the data row by row */
   PLVtmr.capture;
   FOR rec IN (SELECT * FROM parts)
   LOOP
      pnums(SQL%ROWCOUNT)  := rec.partnum;
      pnames(SQL%ROWCOUNT) := rec.partname;
   END LOOP;
   PLVtmr.show_elapsed ('Single row fetch '|| num);

   /* Clean up the in-memory data structures. */   
   pnums.DELETE;
   pnames.DELETE;
   DBMS_SESSION.FREE_UNUSED_USER_MEMORY;
   
   /* Fetch the data in bulk */
   PLVtmr.capture;
   
   SELECT * 
      BULK COLLECT INTO parts_tab -- 11/2002 table of records.
     FROM parts;
     
   p.l ('SQL%ROWCOUNT = ' || SQL%ROWCOUNT);   
   
   PLVtmr.show_elapsed ('BULK COLLECT '|| num);
END;
/

CREATE OR REPLACE PROCEDURE compare_insel_bulk (num IN INTEGER)
IS
   TYPE NumTab IS TABLE OF parts.partnum%TYPE INDEX BY BINARY_INTEGER;
   TYPE NameTab IS TABLE OF parts.partname%TYPE INDEX BY BINARY_INTEGER;
   pnums NumTab;
   pnames NameTab;
   
   TYPE parts_tabtype IS TABLE OF parts%ROWTYPE 
      INDEX BY BINARY_INTEGER;
   parts_tab parts_tabtype;
   
   l_limit PLS_INTEGER := 100;
BEGIN
   EXECUTE IMMEDIATE 'TRUNCATE TABLE parts';
   
   /* Load up the table. */
   FOR indx IN 1..num
   LOOP
      INSERT INTO parts VALUES (indx, 'Part ' || TO_CHAR(indx));
   END LOOP;
   COMMIT;
   
   DBMS_SESSION.FREE_UNUSED_USER_MEMORY;
   
   PLVtmr.capture;
   INSERT INTO parts2
     SELECT * FROM parts;
   PLVtmr.show_elapsed ('Insert Select');

   EXECUTE IMMEDIATE 'TRUNCATE TABLE parts2';
   DBMS_SESSION.FREE_UNUSED_USER_MEMORY;
   
   PLVtmr.capture;
   
   SELECT * 
      BULK COLLECT INTO parts_tab 
     FROM parts;

   FORALL indx IN parts_tab.FIRST .. parts_tab.LAST
      INSERT INTO parts2 VALUES parts_tab(indx);
	  
   PLVtmr.show_elapsed ('BULK COLLECT - FORALL');     
END;
/

BEGIN
   compare_insel_bulk (10000);
   compare_inserting (10000);   
   compare_fetching (100000);
END;
/   

/* Some results...

Procedure created.

.FOR loop 1000 Elapsed: .21 seconds.
.FORALL 1000 Elapsed: .01 seconds.

.FOR loop 10000 Elapsed: 5.68 seconds.
.FORALL 10000 Elapsed: .15 seconds.

PL/SQL procedure successfully completed.


Procedure created.

Input truncated to 4 characters
.Single row fetch 1000 Elapsed: .06 seconds.
.BULK COLLECT 1000 Elapsed: .01 seconds.

.Single row fetch 10000 Elapsed: .59 seconds.
.BULK COLLECT 10000 Elapsed: .16 seconds.

Insert Bulk Select 

10000 rows

Insert Select Elapsed: .05 seconds.
BULK COLLECT - FORALL Elapsed: .1 seconds.

100000 rows

Insert Select Elapsed: .38 seconds.
BULK COLLECT - FORALL Elapsed: 1.02 seconds.

*/
