SET SERVEROUTPUT ON SIZE 1000000 FORMAT WRAPPED

SET ECHO ON

/*

   Program Name: Make Stored Program Units (for this schema)

   Module Name : make.sql

   Written By  : Daniel J. Clamage

   Description : This anonymous PL/SQL block captures all

                 program units (functions, procedures, packages

                 and package bodies) in each their own script file.

                 Each program unit script file includes logging the

                 object compilation to a log file, checking for

                 compilation errors, and any grants.

                 It can be used to easily move all program units for

                 a schema between databases, and works for all 7.x

                 databases and above.

                 HOW IT WORKS:

                 This program generates a script (MAKECODE.SQL)

                 which when run, dumps each program unit's source code

                 into separate files (named OBJxxxxx.SQL where xxxxx

                 is a sequential numbering of the program units).

                 Another script (MAKEALL.SQL) is created which when

                 run, will run all the generated source code scripts.

                 It includes a call to the recompile utility to

                 correct any program unit call dependencies.

                 NOTE: makes use of the system utilities:

                       PUT_LONG_LINE

                       RECOMPILE

                       REV_ENG

   Modification:

     V.001 12-DEC-1998 - djc - Initial release.

     V.002 21-DEC-1998 - djc - Added grants.

     V.003 09-JAN-1999 - djc - Supply target path for files at runtime.

     V.004 16-JAN-1999 - djc - Added spool file for recompile.

     V.005 24-MAY-1999 - djc - Generate filenames by incremental number

                         (OBJxxxxx.SQL) or using program unit type, name

                         (e.g., BLICK.SQH for Package Header Blick).

                         Turned on serveroutput for each module.

*/

ACCEPT path PROMPT 'Enter path (no ending slash) where to write files: '

ACCEPT fids PROMPT 'Generate filename or use program unit name? ([G]|P): '

SET ECHO OFF

SET TERMOUT OFF

SET LINESIZE 200

SET PAGESIZE 0

SET ARRAYSIZE 5

SET HEADING OFF

SET TRIMSPOOL ON

SET VERIFY OFF

SET FEEDBACK OFF

SPOOL &&path\MAKECODE.SQL

DECLARE

  -- constants

  PATHDIR CONSTANT VARCHAR2(50) := '&&path';  -- where to write files to

  LINESZ  CONSTANT NUMBER := 200;             -- match SQL*Plus LINESIZE parameter

  LF      CONSTANT VARCHAR2(1) := CHR(10);    -- newline character

  FIDS    CONSTANT VARCHAR2(1) := UPPER(NVL('&&fids','G'));  -- how to make filename


  -- cursors

  CURSOR get_code IS

  SELECT object_name, object_type

  FROM user_objects

  WHERE object_type IN ('FUNCTION', 'PROCEDURE', 'PACKAGE', 'PACKAGE BODY')

  ORDER BY object_name, object_type;


  -- variables

  lcv NUMBER := 0;     -- used for object file numbering (generation method)


  -- subprograms

  -- generate file names using OBJ and sequential number

  FUNCTION autogen_name(Pnext_id IN NUMBER, Plog IN BOOLEAN DEFAULT FALSE)

  RETURN VARCHAR2 IS

    suffix VARCHAR2(4);  -- file type

  BEGIN

    IF (Plog) THEN  -- set suffix to .LOG

      suffix := '.LOG';

    ELSE  -- set suffix to .SQL

      suffix := '.SQL';

    END IF;  -- set suffix

    RETURN(PATHDIR || '\OBJ' || LTRIM(TO_CHAR(Pnext_id, '00000')) || suffix);

  END autogen_name;


  -- generate file names using unit type and unit name

  FUNCTION gen_name_by_type(

    Pcode_rec IN get_code%ROWTYPE,

    Plog IN BOOLEAN DEFAULT FALSE)

  RETURN VARCHAR2 IS

    suffix VARCHAR2(4);

  BEGIN

    IF (Plog) THEN  -- set suffix to .LOG

      suffix := '.LOG';

    ELSE  -- suffix reflects program unit type

      IF (Pcode_rec.object_type IN ('PROCEDURE', 'FUNCTION')) THEN

        suffix := '.SQL';  -- stand-alone subprogram

      ELSIF (Pcode_rec.object_type = 'PACKAGE') THEN

        suffix := '.SQH';  -- package specification

      ELSIF (Pcode_rec.object_type = 'PACKAGE BODY') THEN

        suffix := '.SQB';  -- package implementation

      ELSE  -- never happens

        suffix := '.SQL';  -- unknown

      END IF;  -- suffix based on program unit type

    END IF;  -- set suffix

    RETURN(PATHDIR || '\' || Pcode_rec.object_name || suffix);

  END gen_name_by_type;


BEGIN

  FOR i IN get_code LOOP  -- get source code

    lcv := lcv + 1;  -- make next object file name

    IF (FIDS = 'G') THEN  -- automatically generate as OBJxxxxx.SQL

      put_long_line('SPOOL ' || autogen_name(lcv), LINESZ);    -- module filename

      put_long_line(

        'EXEC put_long_line(''SET SERVEROUTPUT ON SIZE 1000000 FORMAT WRAPPED'')', LINESZ);

      put_long_line('EXEC put_long_line(''SPOOL ' ||

        autogen_name(lcv, TRUE) || ''');', LINESZ);             -- log filename

    ELSE  -- use program unit name as filename

      put_long_line('SPOOL ' || gen_name_by_type(i), LINESZ);  -- module filename

      put_long_line(

        'EXEC put_long_line(''SET SERVEROUTPUT ON SIZE 1000000 FORMAT WRAPPED'')', LINESZ);

      put_long_line('EXEC put_long_line(''SPOOL ' ||

        gen_name_by_type(i, TRUE) || ''');', LINESZ);           -- log filename

    END IF;  -- how to generate filename

    put_long_line('EXEC put_long_line(''CREATE OR REPLACE'');', LINESZ);

    put_long_line('SELECT text FROM user_source ' || LF || 'WHERE name=''' ||

                  i.object_name || ''' AND type=''' || i.object_type || '''' || LF ||

                  'ORDER BY name, type, line;', LINESZ);

    put_long_line('EXEC put_long_line(''/'');', LINESZ);

    put_long_line('EXEC put_long_line(''EXEC SHOWERR;'');', LINESZ);  -- display any errors

    IF (i.object_type != 'PACKAGE BODY') THEN  -- don't do grants for package body

      put_long_line('EXEC rev_eng.rev_grants(''' || USER || ''', ''' ||

                                             i.object_name || ''');', LINESZ);

    END IF;  -- only do grants for specific object types

    put_long_line('EXEC put_long_line(''SPOOL OFF'');', LINESZ);  -- inside the obj script

    put_long_line('SPOOL OFF', LINESZ);  -- inside makecode script

  END LOOP;  -- get source code


  -- now build script to run all the object scripts (1 - #scripts)

  put_long_line('SPOOL ' || PATHDIR || '\MAKEALL.SQL', LINESZ);

  put_long_line('BEGIN', LINESZ);

  IF (FIDS = 'G') THEN  -- automatically generate as OBJxxxxx.SQL

    FOR i IN 1..lcv LOOP

      put_long_line('put_long_line(''@@' || autogen_name(i) || ''');', LINESZ);

    END LOOP;

  ELSE  -- use program unit name as filename

    FOR i IN get_code LOOP  -- get file name, type

      put_long_line('put_long_line(''@@' || gen_name_by_type(i) || ''');', LINESZ);

    END LOOP;  -- get file name, type

  END IF;  -- how to generate filename


  -- create string to create string to generate recompile block

  put_long_line('put_long_line(''SPOOL ' || PATHDIR || '\recompile.log'');', LINESZ);

  put_long_line('put_long_line(''EXEC put_long_line(''''RECOMPILE: '''' || recompile);'', ' ||

                TO_CHAR(LINESZ) || ');', LINESZ);

  put_long_line('put_long_line(''SPOOL OFF'');', LINESZ);

  put_long_line('END;' || CHR(10) || '/', LINESZ);

  put_long_line('SPOOL OFF', LINESZ);

END;

/


SPOOL OFF

@@&&path\MAKECODE.SQL

SET ECHO ON

SET TERMOUT ON

SET FEEDBACK ON

SET HEADING ON

SET TRIMSPOOL ON

SET VERIFY ON

--@@&&path\MAKEALL.SQL

