CREATE OR REPLACE FUNCTION stockpivot_nopl (dataset refcur_pkg.refcur_t)
   RETURN tickertypeset
IS
   out_obj    tickertype        := tickertype (NULL, NULL, NULL, NULL);
   --
   TYPE dataset_tt IS TABLE OF stocktable%ROWTYPE
      INDEX BY PLS_INTEGER;

   l_dataset   dataset_tt;
   retval     tickertypeset     := tickertypeset ();
   l_row pls_integer;
BEGIN
   FETCH dataset
   BULK COLLECT INTO l_dataset;

   l_row := l_dataset.FIRST;

   WHILE (l_row IS NOT NULL)
   LOOP
      out_obj.ticker := l_dataset(l_row).ticker;
      out_obj.pricetype := 'O';
      out_obj.price := l_dataset(l_row).open_price;
      out_obj.pricedate := l_dataset(l_row).trade_date;
      retval.EXTEND;
      retval (retval.LAST) := out_obj;
	  --
      out_obj.pricetype := 'C';
      out_obj.price := l_dataset(l_row).close_price;
      out_obj.pricedate := l_dataset(l_row).trade_date;
      retval.EXTEND;
      retval (retval.LAST) := out_obj;
	  --
      l_row := l_dataset.NEXT (l_row);
   END LOOP;

   CLOSE dataset;

   RETURN retval;
END;
/

SHO ERR

/* Pivot data with pipelining */

CREATE OR REPLACE FUNCTION stockpivot_pl (dataset refcur_pkg.refcur_t)
   RETURN tickertypeset PIPELINED
IS
   l_row_as_object   tickertype        := tickertype (NULL, NULL, NULL, NULL);

   TYPE dataset_tt IS TABLE OF dataset%ROWTYPE
      INDEX BY PLS_INTEGER;

   l_dataset   dataset_tt;
   retval     tickertypeset     := tickertypeset ();
   l_row pls_integer;
BEGIN
   FETCH dataset
   BULK COLLECT INTO l_dataset;
   CLOSE dataset;

   l_row := l_dataset.FIRST;

   WHILE (l_row IS NOT NULL)
   LOOP
      -- first row
      l_row_as_object.ticker := l_dataset(l_row).ticker;
      l_row_as_object.pricetype := 'O';
      l_row_as_object.price := l_dataset(l_row).open_price;
      l_row_as_object.pricedate := l_dataset(l_row).trade_date;
	  
      PIPE ROW (l_row_as_object);

      -- second row
      l_row_as_object.pricetype := 'C';
      l_row_as_object.price := l_dataset(l_row).close_price;
      l_row_as_object.pricedate := l_dataset(l_row).trade_date;
	  
      PIPE ROW (l_row_as_object);
	  
	  l_row := l_dataset.NEXT (l_row);
   END LOOP;

   RETURN;
END;
/

REM Compare performance using "ROWNUM <" to demonstrate the reality
REM of row piping.

DECLARE
   PROCEDURE init
   IS
   BEGIN
      DELETE FROM tickertable;

      COMMIT;
      DBMS_SESSION.free_unused_user_memory;
      PLVtmr.capture;
   END;
   
   PROCEDURE first_ten_rows_pl
   IS
   BEGIN
      init;

      INSERT INTO tickertable
         SELECT *
           FROM TABLE (stockpivot_pl (CURSOR (SELECT *
                                                FROM stocktable)))
          WHERE ROWNUM < 10;

      PLVtmr.show_elapsed ('Pipelining first 9 rows');
   END;

   PROCEDURE first_ten_rows_nopl
   IS
   BEGIN
      init;

      INSERT INTO tickertable
         SELECT *
           FROM TABLE (stockpivot_nopl (CURSOR (SELECT *
                                                  FROM stocktable)))
          WHERE ROWNUM < 10;

      PLVtmr.show_elapsed ('NO pipelining first 9 rows');
   -- showtabcount ('tickertable');
   END;
BEGIN
   -- Compare "first 9 rows" performance
   first_ten_rows_pl;
   first_ten_rows_nopl;
/*
Some results:

On Oracle9i...

Pipelining first 9 rows Elapsed: .1 seconds.
NO pipelining first 9 rows Elapsed: 10.13 seconds.

On Oracle10g...

Pipelining first 9 rows Elapsed: .1 seconds.
NO pipelining first 9 rows Elapsed: 3.9 seconds.

Pipelining first 9 rows Elapsed: 0 seconds.
NO pipelining first 9 rows Elapsed: 3.58 seconds.

Pipelining first 9 rows Elapsed: 0 seconds.
NO pipelining first 9 rows Elapsed: 3.55 seconds.

And with bulk collect....

Pipelining first 9 rows Elapsed: .02 seconds.
NO pipelining first 9 rows Elapsed: .17 seconds.
*/
END;
/
